@@ -440,6 +440,8 @@ init_gnome_version_stuff (void)
 	if (!xml_file)
 		return;
 	doc = xmlParseFile (xml_file);
+	g_free (xml_file);
+
 	if (!doc)
 		return;
 
@@ -706,6 +708,7 @@ main (int argc, char *argv[])
 	if (druid_data.need_to_download) {
 		int response;
 		if (gtk_dialog_run (GTK_DIALOG (GET_WIDGET ("update-dialog"))) == GTK_RESPONSE_OK) {
+			gtk_widget_destroy (GET_WIDGET ("update-dialog"));
 			start_bugzilla_download ();
 			response = gtk_dialog_run (GTK_DIALOG (GET_WIDGET ("progress-dialog")));
 			if (response == GTK_RESPONSE_CANCEL) {
@@ -713,6 +716,7 @@ main (int argc, char *argv[])
 				gtk_widget_destroy(GET_WIDGET ("progress-dialog"));
 				druid_data.download_in_progress = FALSE;
 			} else if (response == DOWNLOAD_FINISHED_ZERO_RESPONSE) {
+				gtk_widget_destroy(GET_WIDGET ("progress-dialog"));
 				w = gtk_message_dialog_new (NULL,
 							    0,
 							    GTK_MESSAGE_ERROR,
@@ -724,25 +728,12 @@ main (int argc, char *argv[])
 						 		GTK_RESPONSE_OK);
 				gtk_dialog_run (GTK_DIALOG (w));
 				gtk_widget_destroy (w);
-
-				gtk_widget_destroy(GET_WIDGET ("progress-dialog"));
 			} else if (response == DOWNLOAD_FINISHED_OK_RESPONSE) {
-				w = gtk_message_dialog_new (NULL,
-							    0,
-							    GTK_MESSAGE_INFO,
-							    GTK_BUTTONS_OK,
-							    _("Bug Buddy updated its bug information correctly."));
-					      
-				gtk_dialog_set_default_response (GTK_DIALOG (w),
-						 		GTK_RESPONSE_OK);
-				gtk_dialog_run (GTK_DIALOG (w));
-				gtk_widget_destroy (w);
-
 				gtk_widget_destroy(GET_WIDGET ("progress-dialog"));
 			}
+		} else {
+			gtk_widget_destroy (GET_WIDGET ("update-dialog"));
 		}
-		
-		gtk_widget_destroy (GET_WIDGET ("update-dialog"));
 	}
 	
 	load_bugzilla_xml ();
