@@ -175,6 +175,7 @@ load_products_xml (BugzillaBTS *bts, xmlDoc *doc)
 	BugzillaProduct *prod;
 	BugzillaComponent *comp;
 	xmlNode *node, *cur;
+	xmlChar *tmp;
 
 	d(g_print ("products:\n"));
 
@@ -189,22 +190,32 @@ load_products_xml (BugzillaBTS *bts, xmlDoc *doc)
 
 			prod->name        = XML_NODE_GET_PROP (node, "name");
 			prod->description = XML_NODE_GET_PROP (node, "description");
+			prod->versions    = NULL;
 			prod->components  = g_hash_table_new (g_str_hash, g_str_equal);
 
 			bugzilla_bts_insert_product (bts, prod);
 
 			for (cur = node->children; cur; cur = cur->next) {
 				d(g_print ("\t\t%s\n", node->name));
-				if (strcmp (cur->name, "component"))
-					continue;
-				
-				comp = g_new0 (BugzillaComponent, 1);
-				comp->product = prod;
-				
-				comp->name        = XML_NODE_GET_PROP (cur, "value");
-				comp->description = XML_NODE_GET_PROP (cur, "description");
+				if (!strcmp (cur->name, "component")) {
+					comp = g_new0 (BugzillaComponent, 1);
+					comp->product = prod;
+					
+					/* This is a workaround for bad generated products.xml
+					   We really need to strict validation on bugzilla side
+					   or rewrote the script generating products.xml*/
+					tmp = xmlGetProp (cur, "value");
+					if (tmp) {
+						xmlFree (tmp);
+						continue;
+					}
+					comp->name        = XML_NODE_GET_PROP (cur, "value");
+					comp->description = XML_NODE_GET_PROP (cur, "description");
 
-				bugzilla_product_insert_component (prod, comp);
+					bugzilla_product_insert_component (prod, comp);
+				} else if (!strcmp (cur->name, "version")) {
+					prod->versions = g_slist_append(prod->versions, XML_NODE_GET_PROP (cur, "value"));
+				}
 			}
 		}
 	}
@@ -346,6 +357,7 @@ get_xml_file (BugzillaBTS *bts, const char *key, char md5[33], XMLFunc parse_fun
 	xmlfile->tmp_path    = g_build_filename (tmppath,       "bugzilla", bts->subdir, tmp_key, NULL);
 	localdir             = g_build_filename (tmppath,       "bugzilla", bts->subdir, NULL);
 
+	g_free (tmp_key);
 	g_free (tmppath);
 	
 	if (e_mkdir_hier (localdir, S_IRWXU)) {
@@ -354,12 +366,16 @@ get_xml_file (BugzillaBTS *bts, const char *key, char md5[33], XMLFunc parse_fun
 	
 	g_free (localdir);
 
-	if (!bts->md5s_downloaded || !md5 || md5=="")
+	if (!bts->md5s_downloaded || !md5 || md5=="") {
+		g_free (src_uri);
 		return xmlfile;
+	}
 
 	md5_get_digest_from_file (xmlfile->cache_path, digest);
-	if (md5str_equal_digest(md5, digest))
+	if (md5str_equal_digest(md5, digest)) {
+		g_free (src_uri);
 		return xmlfile;
+	}
 		
 	xmlfile->source_uri = gnome_vfs_uri_new (src_uri);
 	g_free (src_uri);
@@ -552,7 +568,7 @@ static xmlDoc *
 load_bugzilla_xml_file (BugzillaXMLFile *xml_file)
 {
 	xmlDoc *doc = NULL;
-	struct stat statbuf;
+	struct stat statbuf, statbuf2;
 
 	if (!stat (xml_file->tmp_path, &statbuf) && statbuf.st_size) {
 		if (!rename (xml_file->tmp_path, xml_file->cache_path)) {
@@ -561,7 +577,11 @@ load_bugzilla_xml_file (BugzillaXMLFile *xml_file)
 	}
 
 
-	if (!doc)
+	if (!doc && !stat (xml_file->cache_path, &statbuf) && !stat(xml_file->system_path, &statbuf2) &&
+	    difftime (statbuf.st_mtime,statbuf2.st_mtime) > 0) 
+		doc = xmlParseFile (xml_file->cache_path);
+	
+	if (!doc) 
 		doc = xmlParseFile (xml_file->system_path);
 
 	return doc;
@@ -931,7 +951,7 @@ load_bugzillas (void)
 	d(g_list_foreach (druid_data.dldests, (GFunc)p_string, NULL));
 }
 
-#define ALL_APPLICATIONS_URI "all-applications:///"
+#define ALL_APPLICATIONS_URI "applications-all-users:///"
 #define APPLET_REQUIREMENTS \
 	"has_all (repo_ids, ['IDL:Bonobo/Control:1.0'," \
 	"		     'IDL:GNOME/Vertigo/PanelAppletShell:1.0']) && " \
@@ -1198,6 +1218,13 @@ update_severity (GtkWidget *w, gpointer data)
 	druid_data.severity = (char *)data;
 }
 
+static void
+update_version (GtkWidget *w, gpointer data)
+{
+	druid_data.version = (char *)data;
+}
+
+
 static void
 add_severity (char *s, GtkMenu *m)
 {
@@ -1250,6 +1277,32 @@ add_application (BugzillaApplication *app, GtkListStore *store)
 			    -1);
 }
 
+void
+update_version_menu (BugzillaProduct *prod)
+{
+	GSList *versions;
+	GtkWidget *w;
+	GtkWidget *m, *c;
+
+	versions = prod->versions;
+
+	m = gtk_menu_new ();
+	c = GET_WIDGET ("the-version-menu");
+        gtk_option_menu_set_menu (GTK_OPTION_MENU (c), m);
+
+	while (versions != NULL) {
+		w = gtk_menu_item_new_with_label ((gchar*)versions->data);
+		g_signal_connect (G_OBJECT (w), "activate",
+				  G_CALLBACK (update_version),
+				  versions->data);
+		gtk_widget_show (w);
+		gtk_menu_shell_append (GTK_MENU_SHELL (m), w);
+		versions = versions->next;
+	}
+}
+
+
+
 static gboolean
 unselect (gpointer data)
 {
@@ -1424,7 +1477,7 @@ generate_email_text (gboolean include_headers)
 	email = g_string_new (NULL);
 
 	subject    = buddy_get_text ("desc-subject");
-	version    = buddy_get_text ("the-version-entry");
+	version    = druid_data.version ? druid_data.version : strdup("unspecified");
 
 	s          = buddy_get_text ("desc-text");
 	body = format_for_width (s);
@@ -1444,6 +1497,8 @@ generate_email_text (gboolean include_headers)
 		g_string_append_printf (email, "Distribution: %s\n", druid_data.distro);
 	if (!is_bugzilla) {
 		g_string_append (email, "(This bug report was generated by Bug Buddy " VERSION ")\n");
+		g_string_append (email, "Description:\n");
+		
 	} else if (druid_data.product->bts->submit_type == BUGZILLA_SUBMIT_DEBIAN) {
 		/* This is what gnome.org and ximian.com use */
 		g_string_append_printf (email, "Package: %s\n", druid_data.product->name);
@@ -1454,7 +1509,7 @@ generate_email_text (gboolean include_headers)
 					: "",
 					version);
 		if (druid_data.gnome_distributor)
-			g_string_append_printf (email, "Gnome distributor: %s\n", druid_data.gnome_distributor);
+			g_string_append_printf (email, "Gnome-Distributor: %s\n", druid_data.gnome_distributor);
 		g_string_append_printf (email, "Synopsis: %s\n", subject);
 		g_string_append_printf (email, "Bugzilla-Product: %s\n", druid_data.product->name);
 		g_string_append_printf (email, "Bugzilla-Component: %s\n", druid_data.component->name);
